#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"


//--== on_open1_activate() ==--------------------------------------------------
// Displays open file chooser dialog to load data file
void
on_load_data1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    // Set app pointers
    app_wnd = lookup_widget(GTK_WIDGET(menuitem), "window1");
    statusbar1 = lookup_widget(GTK_WIDGET(menuitem), "statusbar1");
    get_widgets(menuitem);
    // Display file chooser
    GtkWidget *filechooserdialog1 = create_filechooserdialog1();
    gtk_widget_show (filechooserdialog1);
}

//--== on_save_txt1_activate() ==----------------------------------------------
// Displays save file chooser dialog to save search results as .txt
void
on_save_txt1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    // Set status bar pointer
    if(statusbar1 == NULL);
        statusbar1 = lookup_widget(GTK_WIDGET(menuitem), "statusbar1");
    // Display file chooser, set file type, set title
    GtkWidget *filechdlg2 = create_filechooserdialog2();
    save_as = SAVE_TXT;
    gtk_widget_show (filechdlg2);
    gtk_window_set_title(GTK_WINDOW(filechdlg2), 
        "Save vStar Search Results as Text");
}

//--== on_save_csv1_activate() ==----------------------------------------------
// Displays save file chooser dialog to save search results as .csv
void
on_save_csv1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    // Set status bar pointer
    if(statusbar1 == NULL);
        statusbar1 = lookup_widget(GTK_WIDGET(menuitem), "statusbar1");
    // Display file chooser, set file type, set title
    GtkWidget *filechdlg2 = create_filechooserdialog2();
    save_as = SAVE_CSV;
    gtk_widget_show (filechdlg2);
    gtk_window_set_title(GTK_WINDOW(filechdlg2), 
        "Save vStar Search Results as Comma Separated Values");
}

//--== on_quit1_activate() ==--------------------------------------------------
// Quit the app
void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gtk_main_quit();
}

//==-- on_about1_activate() ==-------------------------------------------------
// Display the about box
void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* aboutdialog1 = create_aboutdialog1();
    gtk_widget_show(aboutdialog1);
}

//--== on_button_filech1_cancel_clicked() ==-----------------------------------
// Cancel the open file chooser
void
on_button_filech1_cancel_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

//--== on_button_filech1_open_clicked() ==-------------------------------------
// Attempt to open the selected file as a variable star data file
void
on_button_filech1_open_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
    gchar wintitle[128] = "vStar - ";
    gchar filename[512] = "";
    gchar* file;
    int rec_loaded;
    // Set pointer to self
    GtkWidget* this_file_dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));

    // Get the selected file name, display it in the window title
    strcat(filename, gtk_file_chooser_get_filename
            (GTK_FILE_CHOOSER(this_file_dlg)));
    file = (strrchr(filename, '/')) + 1;
    strcat(wintitle, file);
    gtk_window_set_title(GTK_WINDOW(app_wnd), wintitle);

    // Call initRecords() first time only
    if(!recs_initd)
    {
        initRecords();
        recs_initd = YES;
    }

    // Load file, test results
    rec_loaded = loadRecords(filename);  
    if(rec_loaded > 0)
	{
        enable_widgets(0, 3, TRUE);

        // Get check box states
       	int enable = 
            gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[12]));
        int enable_mag = 
            gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[4]));
        int enable_per = 
            gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[7]));
        // Enable/disable widgets based on state
        enable_widgets(4, 9, (gboolean)!enable);
        if(!enable_mag)
            enable_widgets(5, 6, (gboolean)enable_mag);
        if(!enable_per)
            enable_widgets(8, 9, (gboolean)enable_per);

        enable_widgets(10, 12, TRUE);
    }
    else 
        enable_widgets(0, 12, FALSE);

    // Clear the store when loading a new data file so grid is empty
    if(store != NULL)
	    gtk_list_store_clear(store);

    // Display number of valid records loaded in status bar
    sprintf(rec_loaded_text, "Records Loaded: %d", rec_loaded);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar1), 1, rec_loaded_text);

    gtk_widget_destroy(this_file_dlg);
}

//--== on_button_search_asc_clicked() ==---------------------------------------
void
on_button_search_asc_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
    do_search(button, 1); // Call do_search() with ascending flag on
}

//--== on_button_search_desc_clicked() ==--------------------------------------
void
on_button_search_desc_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
    do_search(button, 0); // Call do_search() with ascending flag off
}

// v1.01: Enhanced search
//--== on_checkbutton_mag_released() ==----------------------------------------
// Enable magnitude search criteria when checked, else disable
void
on_checkbutton_mag_released            (GtkButton       *button,
                                        gpointer         user_data)
{
    // Get check box state
    int enable = 
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[4]));
    // Enable/disable widgets based on state
    enable_widgets(5, 6, (gboolean)enable);
}

// v1.01: Enhanced search
//--== on_checkbutton_per_released() ==----------------------------------------
// Enable period search criteria when checked, else disable
void
on_checkbutton_per_released            (GtkButton       *button,
                                        gpointer         user_data)
{
    // Get check box state
    int enable = 
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[7]));
    // Enable/disable widgets based on state
    enable_widgets(8, 9, (gboolean)enable);
}

//--== on_checkbutton_spec_released() ==---------------------------------------
// Disable variable search criteria when "No Light Curve" checked, else enable
void
on_checkbutton_spec_released               (GtkButton       *button,
                                        gpointer         user_data)
{
    // Get check box states
    int enable = 
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[12]));
    int enable_mag = 
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[4]));
    int enable_per = 
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[7]));
    // Enable/disable widgets based on state
    enable_widgets(4, 9, (gboolean)!enable);
    if(!enable_mag)
    	enable_widgets(5, 6, (gboolean)enable_mag);
    if(!enable_per)
    	enable_widgets(8, 9, (gboolean)enable_per);
}

//--== on_button_filech2_cancel_clicked() ==-----------------------------------
// Cancel the save file chooser
void
on_button_filech2_cancel_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

//==-- on_button_filech2_save_clicked() ==-------------------------------------
// Attempt to save a file in native variable star data format or as a 
// comma-separated file based on which menu item calls this func
void
on_button_filech2_save_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
    gchar filename[512] = "";
    gchar stattext[128] = "";
    gchar* file;
    int rec_saved;
    // Set pointer to self
    GtkWidget* this_file_dlg = gtk_widget_get_toplevel(GTK_WIDGET(button));

    // Get the file name if entered, else set rec_saved to 0
    if(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(this_file_dlg)) != NULL)
    {
        // Get the file name from the path string
        strcat(filename, gtk_file_chooser_get_filename
                (GTK_FILE_CHOOSER(this_file_dlg)));
        file = (strrchr(filename, '/')) + 1;

	// Test for file type
        if(save_as == SAVE_TXT)
            rec_saved = saveFile(filename);
        else if(save_as == SAVE_CSV)
            rec_saved = saveReport(filename);
        else 
            rec_saved = -1;
    }
    else
        rec_saved = 0;

    // Display records saved appended to records loaded in status bar
    sprintf(stattext, "%s Records Saved: %d", rec_loaded_text, rec_saved);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar1), 1, stattext);

    gtk_widget_destroy(this_file_dlg);
}

//--== do_search() ==----------------------------------------------------------
// Set search criteria, perform search ascending or descending
// Call funcs to build list store and populate tree view with data columns
void
do_search(GtkButton* button, int asc)
{
    // Assign app pointers if needed
    if(treeview1 == NULL)
        treeview1 = lookup_widget(GTK_WIDGET(button), "treeview1");
    if(statusbar1 == NULL);
        statusbar1 = lookup_widget(GTK_WIDGET(button), "statusbar1");
    // gchar pointers to hold search criteria values
    const gchar *ra_west, *ra_east, *dec_south, *dec_north,
          *mag_max, *mag_min, *per_min, *per_max;

    int rec_srchd, spec, mag, per, i = 0; 
    gchar stat_text[128]; // status bar text string

    // Get location search criteria values
    ra_west   = gtk_entry_get_text(GTK_ENTRY(wdgt_list[i++]));
    ra_east   = gtk_entry_get_text(GTK_ENTRY(wdgt_list[i++]));
    dec_south = gtk_entry_get_text(GTK_ENTRY(wdgt_list[i++]));
    dec_north = gtk_entry_get_text(GTK_ENTRY(wdgt_list[i++]));
    
    // Get variability search criteria values
    mag = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[i++]));
    mag_max   = gtk_entry_get_text(GTK_ENTRY(wdgt_list[i++]));
    mag_min   = gtk_entry_get_text(GTK_ENTRY(wdgt_list[i++]));
    per = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[i++]));
    per_min   = gtk_entry_get_text(GTK_ENTRY(wdgt_list[i++]));
    per_max   = gtk_entry_get_text(GTK_ENTRY(wdgt_list[i++]));

    i++; // place holder for 1st button widget
    i++; // place holder for 2nd button widget
    spec = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt_list[i++]));

    // If sting conversion to numeric fails, atoi() or atof() returns zero
    // Set search criteria values via validation functions
    setSchMinHr  (atoi(ra_west));
    setSchMaxHr  (atoi(ra_east));
    setSchMinDeg (atoi(dec_south));
    setSchMaxDeg (atoi(dec_north));
    setSchMaxMag (atof(mag_max));
    setSchMinMag (atof(mag_min));
    setSchMinDays(atof(per_min));
    setSchMaxDays(atof(per_max));

    // Display records search appended to records loaded in status bar
    rec_srchd = searchRecords(!spec, mag, per);
    sprintf(stat_text, "%s Records Searched: %d", rec_loaded_text, rec_srchd);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar1), 1, stat_text);

    // Clear the store if needed
    if(list_store_done)
        gtk_list_store_clear(store);

    // Call func to populate store ascending or descending
    if(rec_srchd > 0)
    {
        store = build_list_store(asc);
        list_store_done = YES;
    }
    else // Set store state to NO if no search records
    {
        list_store_done = NO;
        return;
    }

    // Clear tree view columns if needed
    if(list_cols_done)
        del_list_columns();

    // Call func to populate tree view w/ columns and data
    add_list_columns(store); 
    list_cols_done = YES;
}

// Build the list store from search results
GtkListStore*
build_list_store(int asc) 
{
    gint i = 0;
    GtkListStore *store;
    GtkTreeIter iter;
    char raStr[6];
    char decStr[4];
    char dayStr[16];

    /* create list store */

    // Build columns
    store = gtk_list_store_new (7,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING,
			      G_TYPE_STRING);

    // Navigate asc or desc
    asc ? moveFirst() : moveLast();
    // Populate columns
    while(asc ? !eof() : !bof())
    {
        // Format numeric data as string data for display
        sprintf(raStr, "%02d:%02d", getHour(), getMin());

        if(getDeg() >= 0)
            sprintf(decStr, " %02d", getDeg());
        else
            sprintf(decStr, "-%02d", -getDeg());

        getDays(dayStr);

        // Fill store columns with data
        gtk_list_store_append(store, &iter);
        gtk_list_store_set(store, &iter,
			   0, raStr,
			   1, decStr, 
			   2, getName(),
			   3, getType(),
			   4, getRange(),
			   5, dayStr,
			   6, getDesc(), -1);

        asc ? moveNext() : movePrev();
    }

    return store; // return a pointer to the store
}

//--== add_list_columns() ==---------------------------------------------------
// Add tree view columns and fill with data using a cell renderer
void
add_list_columns(GtkListStore* store)
{
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    //== Can the headers be right aligned where needed? ==//

    // Right Ascension column
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("R.A.", renderer,
						       "text", 0,
						       NULL);
    gtk_tree_view_column_set_sort_column_id (column, 0);
    gtk_tree_view_column_set_resizable (column, 1);
    gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), column);
    // Declination column
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Dec.", renderer,
						       "text", 1,
						       NULL);
    gtk_tree_view_column_set_sort_column_id (column, 1);
    gtk_tree_view_column_set_resizable (column, 1);
    gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), column);
    // Name column
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Name", renderer,
						       "text", 2,
						       NULL);
    gtk_tree_view_column_set_sort_column_id (column, 2);
    gtk_tree_view_column_set_resizable (column, 1);
    gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), column);
    // Type column
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Type", renderer,
						       "text", 3,
						       NULL);
    gtk_tree_view_column_set_sort_column_id (column, 3);
    gtk_tree_view_column_set_resizable (column, 1);
    gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), column);
    // Magnitude range column
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Range", renderer,
						       "text", 4,
						       NULL);
    gtk_tree_view_column_set_sort_column_id (column, 4);
    gtk_tree_view_column_set_resizable (column, 1);
    gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), column);
    // Period column
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Period", renderer,
						       "text", 5,
						       NULL);
    gtk_tree_view_column_set_sort_column_id (column, 5);
    gtk_tree_view_column_set_resizable (column, 1);
    gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), column);
    // Description column
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("Description", renderer,
						       "text", 6,
						       NULL);
    gtk_tree_view_column_set_sort_column_id (column, 6);
    gtk_tree_view_column_set_resizable (column, 1);
    gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), column);
    
    // Assign this store to the treeview widget
    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview1), GTK_TREE_MODEL(store));
}

//--== del_list_columns() ==---------------------------------------------------
// Clear the tree view's columns for a clean slate
void
del_list_columns()
{
    gint i;

    for(i = 6; i >=0; i--)
        gtk_tree_view_remove_column(GTK_TREE_VIEW(treeview1),
            gtk_tree_view_get_column(GTK_TREE_VIEW(treeview1), i));
}

//--== get_widgets() ==--------------------------------------------------------
// Assign pointer values to widget pointer array
void
get_widgets(GtkMenuItem* menuitem)
{
    int i = 0;

    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "entry_ra_west");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "entry_ra_east");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "entry_dec_south");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "entry_dec_north");

    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "checkbutton_mag");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "entry_mag_max");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "entry_mag_min");

    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "checkbutton_per");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "entry_per_min");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "entry_per_max");

    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "button_search_asc");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "button_search_desc");
    wdgt_list[i++] = lookup_widget(GTK_WIDGET(menuitem), "checkbutton_spec");
}

//--== enable_widgets() ==-----------------------------------------------------
// Enable/disable a zero-based range of widgets by passing:
//  - minimum range index
//  - maximum range index
//  - TRUE/FALSE enable state
void
enable_widgets(int min, int max, gboolean enable)
{
    int i;

    for(i = min; i <= max; i++)
        gtk_widget_set_sensitive(wdgt_list[i], enable);
}



